clear
close all
close hidden
warning off all
global theta_est theta_bound lin_param

theta_est = [];
theta_bound = [];

params = dlmread('parameter.txt');
rep = params(1);
seed = RandStream('mcg16807', 'Seed',rep); %set seeds
RandStream.setGlobalStream(seed);
%tuning parameters
alpha = params(2);
sigma = params(3);
M = params(4); %numberof markets
J = params(5); %number of "inside" products in each market
fac = params(6); %denominator of prod_id/fac for generating x
ns = params(7); %number of consumer draws in generating empirical shares

beta = 1;
lin_param = [alpha;beta];
n = M*J;

%market id
mk_id = kron((1:M)',ones(J,1));
mk_ind = (J:J:n)'; %indicator for the last obs in each market
pd_id = kron(ones(M,1),(1:J)'); %product id

%random draws
x = pd_id/fac + randn(n,1);
xi = .1*randn(n,1);

%draw rc on x (quasi-random seed)
q_rc = haltonset(1,'Skip',1e3,'Leap',1e2);
q_rc_tmp = net(q_rc,1000)';
rc_seed = norminv(q_rc_tmp,0,1); %normal draws

rc_sd = sigma*rc_seed;
mu_x = bsxfun(@times,rc_sd,x);

%constant
con = ones(n,1);

%simulate shares
m = zeros(n,1);
out_ch = zeros(n,1);
share_true = zeros(n,1);
i = 1;
for i =1:M
    x_temp = x(mk_id==i);
    delta_temp = alpha + beta*x_temp + xi(mk_id==i);
    mu_temp = mu_x(mk_id==i,:);
    u_temp = bsxfun(@plus,delta_temp,mu_temp);
    sh_i_temp = bsxfun(@rdivide,exp(u_temp),1+sum(exp(u_temp)));
    share_true_tmp = mean(sh_i_temp,2);
    share_true(mk_id==i,1) = share_true_tmp;
    sh_tmp = [share_true_tmp' 1-sum(share_true_tmp)];
    
    m_tmp = mnrnd(ns,sh_tmp)';
    m(mk_id==i,1) = m_tmp(1:end-1);
    out_ch(mk_id==i,1) = ns - sum(m(mk_id==i,1));
    
end
delta_true = alpha + beta*x + xi;
zero_frac = sum(m==0)/n;

%options for optimization
options = optimset(...
    'MaxFunEvals', 1000,...
    'TolFun',      1e-6,...
    'TolX',        1e-6,...
    'MaxIter',     10,...
    'Display','off');

%% BLP estimation
share = m./ns;
share_out = out_ch./ns;
sele_ind = share>0;

%selected dataset
x_sele = x(sele_ind);
share_sele = share(sele_ind);
x1_sele = [ones(sum(sele_ind),1) x_sele];
mk_id_sele = mk_id(sele_ind);

mk_ind_sele = zeros(M,1);
for i = 1:M
    tmp = sum(mk_id_sele == i);
    if i == 1
        mk_ind_sele(i,1) = tmp;
    else
        mk_ind_sele(i,1) = mk_ind_sele(i-1,1) + tmp;
    end
end

%starting value of \xi
xi_sele = xi(sele_ind);
expmvalblp = exp(alpha + beta*x_sele+xi_sele);

%construct IV
IV_blp = [x1_sele (x_sele.^2-1) (x_sele.^3-3*x_sele)];
invA_blp = pinv(IV_blp'*IV_blp);

%% Laplace Shares
%Laplace share
share_lap = (m + 1)/(ns + J + 1);

x1 = [con x];
expmval= exp(delta_true);

%% bounds approach with fixed iotas
%construct g functions and weights
r_n = 50; % the r runs from 1 to r_n
[weig_g, g1] = g_func(x,r_n);
g = [g1 con];
%weights on the constant is equal to some constant bigger than the weight
%of the coarsest hypercube of the x
weig_g(size(g,2)) = 1/202;

%iota's
iota_u = 2;
iota_l = eps;

%do optimization to estimate parameters (weights on different types)
[sigma_bd,fval_bound]=fminsearch(@(a)obj_bound(a,rc_seed,x,expmval,share_lap,mk_id,mk_ind,M,m,ns,iota_u,iota_l,g,weig_g),sigma+.1,options);
theta_bd = theta_bound;

%do bootstrap
Test_bt = NaN(1000,1);
sigma_bd_bt = NaN(1000,1);
theta_bd_bt = NaN(1000,2);
for i = 1:1000
    
    bts_ind = randsample(M,M,true);
    bts = zeros(n,1);
    for ii =1:M
        bts((ii-1)*J+1:ii*J) = find(mk_id == bts_ind(ii));
    end   
    x_bt = x(bts,:);
    m_bt = m(bts,:);
    expmval_bt = expmval(bts,:);
    share_lap_bt = share_lap(bts,:);
    g_bt = g(bts,:);
    [sigma_bd_bt(i),~]=fminsearch(@(a)obj_bound(a,rc_seed,x_bt,expmval_bt,...
        share_lap_bt,mk_id,mk_ind,M,m_bt,ns,iota_u,iota_l,g_bt,weig_g),sigma_bd,options);    
    theta_bd_bt(i,:) = theta_bound';   
 
end

%mean and s.e.
mean_bt = nanmean([sigma_bd_bt theta_bd_bt]);
se_bt = nanstd([sigma_bd_bt theta_bd_bt]);

results = ['zeros_rclogit_bound_bootstrap_', num2str(rep)];
save(results, 'params','zero_frac', 'sigma_bd','theta_bd',...
   'sigma_bd_bt','theta_bd_bt','Test_bt','mean_bt','se_bt');