clear
close all
close hidden
warning off all

params = dlmread('parameter.txt');
rep = params(1);
seed = RandStream('mcg16807', 'Seed',rep); %set seeds
RandStream.setGlobalStream(seed);

%tuning parameters
alpha = params(2);
sigma = params(3);
M = params(4); %numberof markets

%parameters in the utility function
beta = 1;

%number of products in each market
J = 50;
n = M*J;

%number of consumer draws
ns = 10000;

%market/product id
mk_id = kron((1:M)',ones(J,1));
mk_ind = (J:J:n)'; %indicator for the last obs in each market
pd_id = kron(ones(M,1),(1:J)');

%random draws
x_tmp = rand(n,1);
x_cut = [9.9/10; 9.95/10];
x_val = [12; 15];
x = 1*(x_tmp<x_cut(1)) + x_val(1)*(x_tmp>=x_cut(1)&x_tmp<=x_cut(2)) + x_val(2)*(x_tmp>x_cut(2));
xi = (2*(x_tmp<x_cut(1)) + .1*(x_tmp>=x_cut(1))).*randn(n,1);

con = ones(n,1);

%draw rc on x (quasi-random seed)
q_rc = haltonset(1,'Skip',1e3,'Leap',1e2);
q_rc_tmp = net(q_rc,2000)';
rc_seed = norminv(q_rc_tmp,0,1); %normal draws
rc_sd = sigma*rc_seed;
mu_x = bsxfun(@times,rc_sd,x);

%simulate shares
m = zeros(n,1);
out_ch = zeros(n,1);
share_true = zeros(n,1);
i = 1;
for i =1:M
    x_temp = x(mk_id==i);
    delta_temp = alpha + beta*x_temp + xi(mk_id==i);
    mu_temp = mu_x(mk_id==i,:);
    u_temp = bsxfun(@plus,delta_temp,mu_temp);
    sh_i_temp = bsxfun(@rdivide,exp(u_temp),1+sum(exp(u_temp)));
    share_true_tmp = mean(sh_i_temp,2);
    share_true(mk_id==i,1) = share_true_tmp;
    sh_tmp = [share_true_tmp' 1-sum(share_true_tmp)];
    
    m_tmp = mnrnd(ns,sh_tmp)';
    m(mk_id==i,1) = m_tmp(1:end-1);
    out_ch(mk_id==i,1) = ns - sum(m(mk_id==i,1));
    
end
delta_true = alpha + beta*x + xi;
zero_frac = sum(m==0)/n;

%% Empirical Shares
share = m./ns;
share_out = out_ch./ns;
x1 = [x,con];

sele_ind = share>0;
x_sele = x(sele_ind);
xi_sele = xi(sele_ind);
con_sele = con(sele_ind);
share_sele = share(sele_ind);
mk_id_sele1 = mk_id(sele_ind);
[~,mk_ind_sele,mk_id_sele] = unique(mk_id_sele1,'last','legacy');

mu_x_sele = bsxfun(@times,rc_sd,x_sele);
x_ind_sele_exp = exp(mu_x_sele);
expmvalblp = exp(alpha*con_sele + beta*x_sele+xi_sele);
delta_blp = meanval(expmvalblp,share_sele,x_ind_sele_exp,mk_id_sele,mk_ind_sele);
est_blp = regress(delta_blp,x1(sele_ind,:));

%% Laplace shares
share_ls = (m+1)./(ns+J+1);
share_ls1 = (m+1)./ns;

mu_x = bsxfun(@times,rc_sd,x);
x_ind_exp = exp(mu_x);
expmvalbound = exp(alpha + beta*x+xi);

delta_ls = meanval(expmvalbound,share_ls,x_ind_exp,mk_id,mk_ind);
est_ls = regress(delta_ls,x1);

delta_ls1 = meanval(expmvalbound,share_ls1,x_ind_exp,mk_id,mk_ind);
est_ls1 = regress(delta_ls1,x1);

%% Bounds Approach
g = [x==1 x~=1 x==x_val(1) x~=x_val(1) x==x_val(2) x~=x_val(2) con 1-con];

%iota's
iota_u = 2;
iota_l = eps;

%use original Laplace share
delta_u = delta_ls - log(share_ls) + log((m+iota_u)./ns);
delta_l = delta_ls - log(share_ls) + log((m+iota_l)./ns);

y_u = delta_u;
y_l = -delta_l;
Y_sum = g'*[y_l y_u];
X_sum = g'*[-x1 x1];

param_0 = [beta+.1*rand;alpha];
[bound_est,~] = test_stat((1/(M^0.5))*Y_sum,(1/(M^0.5))*X_sum,param_0);

%use ls1
delta_u1 = delta_ls1 - log(share_ls1) + log((m+iota_u)./ns);
delta_l1 = delta_ls1 - log(share_ls1) + log((m+iota_l)./ns);

y_u = delta_u1;
y_l = -delta_l1;
Y_sum = g'*[y_l y_u];
X_sum = g'*[-x1 x1];

[bound_est1,~] = test_stat((1/(M^0.5))*Y_sum,(1/(M^0.5))*X_sum,param_0);

results = ['rclogit_fixrc_crazyzeros_', num2str(rep)];
save(results, 'params','zero_frac', 'est_blp','est_ls','bound_est','bound_est1');
