clear
close all
close hidden
warning off all

params = dlmread('parameter.txt');
rep = params(1);
seed = RandStream('mcg16807', 'Seed',rep); %set seeds
RandStream.setGlobalStream(seed);

%intercept adjustment
inte_adj = params(2);

%iota's
iota_u = 2;
iota_l = eps;

%coefficients on product dummies (and constant): use the estimates in the 
%empirical application as true value
beta_0 = [-2.68891402578341;-0.588552832283657;-1.67084465333921;-1.28599428390645;-1.32904119142626;-0.551707491105780;-1.56438823573588;-2.14856925380612;-1.68708142964880;-3.96654972785554;-0.513088752589182;-3.05538286150021;-3.95853555235837;-2.18342334274988;-3.40989341767348;-2.58847919921996;-1.48382825640209;-4.00315460258500;-6.10878071522482;-0.587380769669832;-1.71540698594899;-2.87667876047626;-1.36981034347657;-0.497106495480725;-1.94108303792369;-4.48501710536335;-1.60910873749585;-1.42940688537483;-1.89862032179382;-3.55308493005387;-4.76808728637298;-0.968101418399197;-3.69212332521348;-0.0314604627625783;-1.25418352277727;-1.97669180740204;-0.601764961409288;-3.50936395549028;-0.792734460643418;-3.81273065241750;-2.53488927008886;-1.47119142563730;-1.67209116798499;-2.72274987244760;-4.70713012545229;-0.990738139652560;-1.30129821625135;-1.52694764843858;-0.570121067166226];
beta_0 = beta_0 - inte_adj;

%random coefficient on price: log-normal with rc_mu and rc_sigma
rc_ln_mu = 0;
rc_ln_sigma = .5;

%----------------load and generate data---------------------------------
load data_allstores_lent
data(data.week<289,:) = [];

n = size(data.mk_id,1);
prod_dumm = data{:,22:70};
prod_dumm_remove_ind = sum(prod_dumm)==0;

beta_0([false prod_dumm_remove_ind(1:end-1)]) = []; 
prod_dumm(:,prod_dumm_remove_ind) = [];

%add a new characteristics for identification of random coefficient
z = randn(n,1);
X = [ones(n,1) prod_dumm(:,1:end-1)];
k_X = size(X,2);

%fixed part in mean utility: not to be estimated
delta_fix = z + X*beta_0;

%generate demand shock \xi
sigma_xi = 1.5*(delta_fix <= median(delta_fix)) + .5*(delta_fix > median(delta_fix));
xi = min(max(sigma_xi.*randn(n,1),-3*sigma_xi),3*sigma_xi);

%generate price
%marginal cost
mc = (100-data.regular_profit).*data.regular_price/100;
%a reduced form price equation: .3 is chosen to match the mean of actual
%price data
price_sim = mc + .3*exp(xi);

%draw rc on price (quasi-random seed)
q_rc = haltonset(1,'Skip',1e3,'Leap',1e2);
rc_seed = net(q_rc,1000)';
rc = exp(norminv(rc_seed,rc_ln_mu,rc_ln_sigma)); %log-normal draws
rc_price = -bsxfun(@times,rc,price_sim);

%simulate market shares
[~,~,mk_id_new] = unique(data.mk_id,'stable');
[mk_id_set,mkindex,mkid] = unique(mk_id_new,'last');
T = length(mk_id_set);
m = zeros(n,1);
out_ch = zeros(n,1);
share_true = zeros(n,1);
delta_true = zeros(n,1);
for i = 1:T
    ind_t = mk_id_new == mk_id_set(i); 
    
    delta_temp = z(ind_t,:) + X(ind_t,:)*beta_0 + xi(ind_t,:);
    rc_temp = rc_price(ind_t,:);
    delta_true(ind_t,:) = delta_temp;
    
    u_temp = bsxfun(@plus,delta_temp,rc_temp);
    sh_i_temp = bsxfun(@rdivide,exp(u_temp),1+sum(exp(u_temp)));
    share_true_tmp = mean(sh_i_temp,2);
    share_true(ind_t,1) = share_true_tmp;
    sh_tmp = [share_true_tmp' 1-sum(share_true_tmp)];
    
    ns = max(data.custcoun(ind_t,:));
    m_tmp = mnrnd(ns,sh_tmp)';
    m(ind_t,1) = m_tmp(1:end-1);
    out_ch(ind_t,1) = ns - sum(m(ind_t,1));
end
zero_frac = sum(data.quantity == 0)/n;
zero_frac_sim = sum(m==0)/n;

%options for optimization
options = optimset(...
    'MaxFunEvals', 1000,...
    'TolFun',      1e-6,...
    'TolX',        1e-6,...
    'MaxIter',     200,...
    'Display','iter');

options2 = optimoptions('fmincon','MaxIter',5,'Display','iter');

%% BLP estimation
share = m./data.custcoun;
sele_ind = share>0;

%selected dataset
z_sele = z(sele_ind);
X_sele = X(sele_ind,:);
delta_fix_sele = delta_fix(sele_ind);
[X_sele, ~] = licols(X_sele,1e-10);

price_sim_sele = price_sim(sele_ind);
mc_sele = mc(sele_ind);
share_sele = share(sele_ind);

%mkid_sele = mkid(sele_ind);
[~,~,mk_id_new_sele] = unique(data.mk_id(sele_ind),'stable');
[~,mkindex_sele,mkid_sele] = unique(mk_id_new_sele,'last');

%starting value of mean utility
expmvalblp = exp(delta_true(sele_ind));

%construct IV
IV_blp = [X_sele mc_sele z_sele];
invA_blp = pinv(IV_blp'*IV_blp);

%do optimization to estimate parameters
LB = [rc_ln_mu-1;rc_ln_sigma-1];
UB = [rc_ln_mu+1;rc_ln_sigma+1];
[rc_para_blp,fval_blp]=fminsearchbnd(@(a)obj_blp(a,price_sim_sele,delta_fix_sele,expmvalblp,rc_seed,IV_blp,invA_blp,share_sele,mkid_sele,mkindex_sele),[rc_ln_mu;rc_ln_sigma],LB,UB,options);

%% Laplace Shares
%Laplace shares
share_lap = (m + 1)./(data.custcoun + data.num_prod + 1);

%starting value of mean utility
expmval = exp(delta_true);

%construct IV
IV_lap = [X mc z];
invA_lap = pinv(IV_lap'*IV_lap);

[rc_para_lap,fval_lap]=fminsearchbnd(@(a)obj_blp(a,price_sim,delta_fix,expmval,rc_seed,IV_lap,invA_lap,share_lap,mkid,mkindex),[rc_ln_mu;rc_ln_sigma],LB,UB,options);

%% bounds approach 
r_s = 10;
r_l = 20;

%continuous variables g function
[w_con_s, g_con_s] = g_func([mc z],r_s);
[w_con_l, g_con_l] = g_func([mc z],r_l);

%interactions-based g functions
r_in = 3;
r1 = (2:2:2*r_in)';
n_g_in = 2*(sum(r1));
n_con_in = n_g_in*k_X;
g_in = zeros(n,n_con_in);
w_g_in = zeros(n_con_in,1);
for k = 2:k_X
    [weig_g_tmp,g_tmp]= g_func([mc.*X(:,k) z.*X(:,k)],r_in);
    g_in(:,(k-1)*n_g_in+1:k*n_g_in) = g_tmp;
    w_g_in((k-1)*n_g_in+1:k*n_g_in) = (2/3)*weig_g_tmp;  
end

%remove zero columns
g_zero_ind = sum(g_in) == 0;
g_in(:,g_zero_ind) = [];
w_g_in(g_zero_ind') = [];

%set I: small r, no interaction with product dummy
g_I = [g_con_s X];
w_g_I = [w_con_s; max(w_con_s)*ones(k_X,1)];

%set II: small r, with interaction with product dummy
g_II = [g_con_s g_in];
w_g_II = [w_con_s; w_g_in];

%set III: large r, no interaction with product dummy
g_III = [g_con_l X];
w_g_III = [w_con_l; max(w_con_l)*ones(k_X,1)];

%set IV: large r, with interaction with product dummy
g_IV = [g_con_l g_in];
w_g_IV = [w_con_l; w_g_in];

[rc_para_bd_I,fval_bd_I]=fminsearchbnd(@(a)obj_bound(a,rc_seed,price_sim,delta_fix,expmval,share_lap,mkid,mkindex,iota_u,iota_l,m,data.custcoun,g_I,w_g_I),[rc_ln_mu;rc_ln_sigma],LB,UB,options);

[rc_para_bd_II,fval_bd_II]=fminsearchbnd(@(a)obj_bound(a,rc_seed,price_sim,delta_fix,expmval,share_lap,mkid,mkindex,iota_u,iota_l,m,data.custcoun,g_II,w_g_II),[rc_ln_mu;rc_ln_sigma],LB,UB,options);

[rc_para_bd_III,fval_bd_III]=fminsearchbnd(@(a)obj_bound(a,rc_seed,price_sim,delta_fix,expmval,share_lap,mkid,mkindex,iota_u,iota_l,m,data.custcoun,g_III,w_g_III),[rc_ln_mu;rc_ln_sigma],LB,UB,options);

[rc_para_bd_IV,fval_bd_IV]=fminsearchbnd(@(a)obj_bound(a,rc_seed,price_sim,delta_fix,expmval,share_lap,mkid,mkindex,iota_u,iota_l,m,data.custcoun,g_IV,w_g_IV),[rc_ln_mu;rc_ln_sigma],LB,UB,options);

results = ['zeros_mc_tuna_', num2str(rep)];
save(results, 'params','zero_frac_sim', ...
    'rc_para_blp','rc_para_lap','rc_para_bd_I','rc_para_bd_II','rc_para_bd_III','rc_para_bd_IV');